"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.onEvent = void 0;
const child_process_1 = require("child_process");
const wait_port_1 = __importDefault(require("wait-port"));
async function onEvent(event) {
    const resourceProps = await extractResourceProperties(event);
    switch (event.RequestType) {
        case 'Create':
        case 'Update':
            waitForTemporal(resourceProps);
            createNamespace(resourceProps);
            return {
                PhysicalResourceId: resourceProps.resourcePhysicalId,
            };
        case 'Delete': {
            // FIXME: Deleting a namespace is not yet supported - https://github.com/temporalio/temporal/issues/1679
            return {
                PhysicalResourceId: resourceProps.resourcePhysicalId,
            };
        }
    }
}
exports.onEvent = onEvent;
async function extractResourceProperties(event) {
    const inputProps = event.ResourceProperties;
    if (!inputProps.TemporalHost)
        throw new Error('"TemporalHost" is required');
    if (!inputProps.NamespaceName)
        throw new Error('"NamespaceName" is required');
    const resourcePhysicalId = `${inputProps.TemporalHost}://${inputProps.NamespaceName}`;
    return {
        temporalHost: inputProps.TemporalHost,
        namespaceName: inputProps.NamespaceName,
        resourcePhysicalId,
    };
}
function waitForTemporal(resourceProps) {
    const [host, port] = resourceProps.temporalHost.split(':');
    (0, wait_port_1.default)({
        host: host,
        port: parseInt(port),
    });
}
function createNamespace(resourceProps) {
    execTctlTool(resourceProps, ['--namespace', resourceProps.namespaceName, 'namespace', 'register']);
}
function execTctlTool(context, command) {
    const args = [];
    const env = {};
    args.push('--address', `${context.temporalHost}`);
    args.push('--auto_confirm');
    try {
        (0, child_process_1.execFileSync)('/opt/temporal/bin/tctl', [...args, ...command], {
            encoding: 'utf-8',
            env,
            stdio: [
                'ignore',
                process.stderr,
                'inherit', // inherit stderr
            ],
        });
    }
    catch (e) {
        (0, child_process_1.execFileSync)('/opt/temporal/bin/tctl', [...args, ...command], {
            encoding: 'utf-8',
            env,
            stdio: [
                'ignore',
                process.stderr,
                'inherit', // inherit stderr
            ],
        });
        // FIXME: Explicitly check for "Already exists" errors.
        //
        // Error: Register namespace operation failed.
        // Error Details: rpc error: code = AlreadyExists desc = Namespace already exists.
        // ERROR	Invoke Error 	{
        //    "errorType": "Error",
        //    "errorMessage": "Command failed: /opt/temporal/bin/tctl --address xxxx:7233 --auto_confirm --namespace xxxx namespace register",
        //    "status": 1,
        //    "signal": null,
        //    "output": [
        //        null,
        //        null,
        //        null
        //    ],
        //    "pid": 25,
        //    "stdout": null,
        //    "stderr": null,
        //    "stack": [
        //        "Error: Command failed: /opt/temporal/bin/tctl --address xxxx:7233 --auto_confirm --namespace xxxx namespace register",
        //        "    at checkExecSyncError (child_process.js:790:11)",
        //        "    at execFileSync (child_process.js:827:15)",
        //        "    at execTctlTool (/var/task/index.js:2658:36)",
        //        "    at createNamespace (/var/task/index.js:2651:3)",
        //        "    at Runtime.onEvent [as handler] (/var/task/index.js:2618:7)"
        //    ]
        //}
    }
}
//# sourceMappingURL=TemporalNamespaceHandler.js.map